@@ -4,11 +4,20 @@ import json
 import codecs
 import inspect
 from django.http import JsonResponse
+from django.utils.decorators import method_decorator
 from django.utils.safestring import mark_safe
+from django.views.decorators.csrf import ensure_csrf_cookie
 
-from ajax_helpers.utils import ajax_command
+from ajax_helpers.utils import ajax_command, is_ajax
 
 
+def ajax_method(func):
+    def method(*args, _ajax=True, **kwargs):
+        return func(*args, **kwargs)
+    return method
+
+
+@method_decorator(ensure_csrf_cookie, name='dispatch')
 class AjaxHelpers:
     ajax_commands = ['button', 'tooltip', 'timer', 'ajax']
 
@@ -32,7 +41,7 @@ class AjaxHelpers:
         return JsonResponse(self.response_commands, safe=False)
 
     def post(self, request, *args, **kwargs):
-        if request.is_ajax():
+        if is_ajax(request):
             if request.content_type == 'application/json':
                 response = json.loads(request.body)
             elif request.content_type == 'multipart/form-data':
@@ -40,11 +49,19 @@ class AjaxHelpers:
             else:
                 response = None
             if response:
-                for t in self.command_set:
-                    if t in response and hasattr(self, f'{t}_{response[t]}'):
-                        method = getattr(self, f'{t}_{response[t]}')
-                        del response[t]
+                if 'ajax_method' in response:
+                    method = getattr(self, response.pop('ajax_method'))
+                    if '_ajax' in inspect.signature(method).parameters:
                         return method(**response)
+                    else:
+                        raise Exception(f'Method {method.__qualname__} is not enabled for ajax. '
+                                        f'Use decorator ajax_method.')
+                else:
+                    for t in self.command_set:
+                        if t in response and hasattr(self, f'{t}_{response[t]}'):
+                            method = getattr(self, f'{t}_{response[t]}')
+                            del response[t]
+                            return method(**response)
         if hasattr(super(), 'post'):
             # noinspection PyUnresolvedReferences
             return super().post(request, *args, **kwargs)
@@ -55,9 +72,13 @@ class AjaxHelpers:
         else:
             self.page_commands.append(ajax_command(function_name, **kwargs))
 
+    def get_page_commands(self):
+        return []
+
     def get_context_data(self, **kwargs):
         # noinspection PyUnresolvedReferences
         context = super().get_context_data(**kwargs) if hasattr(super(), 'get_context_data') else {}
+        self.page_commands += self.get_page_commands()
         if self.page_commands:
             command = ajax_command('onload', commands=self.page_commands)
             context['ajax_helpers_script'] = mark_safe(
@@ -65,11 +86,14 @@ class AjaxHelpers:
             )
         return context
 
+    def dispatch(self, *args, **kwargs):
+        return super().dispatch(*args, **kwargs)
+
 
 class ReceiveForm:
 
     def post(self, request, *args, **kwargs):
-        if request.is_ajax() and request.content_type == 'multipart/form-data':
+        if is_ajax(request) and request.content_type == 'multipart/form-data':
             if 'form_id' in request.POST:
                 return getattr(self, f'form_{request.POST["form_id"]}')(**request.POST.dict())
         # noinspection PyUnresolvedReferences
@@ -99,7 +123,7 @@ class AjaxFileHelpers(AjaxHelpers):
         return JsonResponse({'file': 'ok'})
 
     def post(self, request, *args, **kwargs):
-        if request.is_ajax() and request.content_type == 'multipart/form-data' and 'ajax_file' in request.FILES:
+        if is_ajax(request) and request.content_type == 'multipart/form-data' and 'ajax_file' in request.FILES:
             return self.file_upload(request.FILES['ajax_file'])
         return super().post(request, *args, **kwargs)
 
@@ -116,7 +140,7 @@ class AjaxFileUploadMixin:
 
     # noinspection PyUnresolvedReferences
     def post(self, request, *args, **kwargs):
-        if request.is_ajax() and request.content_type == 'multipart/form-data' and self.upload_key in request.FILES:
+        if is_ajax(request) and request.content_type == 'multipart/form-data' and self.upload_key in request.FILES:
             response = request.POST.dict()
             file = self.request.FILES[self.upload_key]
             upload_params = json.loads(response.get('upload_params', '{}'))
